/*! 
    \file animation3d.h
*/

#include "animation.h"

#include "vector.h"
class PNode;

//! KeyFrame3D Struct
/**
   This represents one point with direction of the animation
*/
typedef struct KeyFrame3D {
  float duration;
  Vector position;
  Quaternion direction;
  ANIM_FUNCTION animFunc;
};

//! Animation Struct
/**
   This represents an animation for a object
*/
class Animation3D : public Animation
{
 public:
  Animation3D(void);
  virtual ~Animation3D(void);
    
  virtual void rewind(void);

  void addKeyFrame(Vector position, Quaternion direction, float time, ANIM_FUNCTION animFunc = ANIM_LINEAR);
  void addKeyFrame(KeyFrame3D* frame);

  virtual void tick(float timePassed);

  // animation functions
  void setAnimFunc(ANIM_FUNCTION animFunc);

 private:
  float constant(float timePassed) const;
  float linear(float timePassed) const;
  float sine(float timePassed) const;
  float cosine(float timePassed) const;
  float exp(float timePassed) const;
  float negExp(float timePassed) const;
  float quadratic(float timePassed) const;
  float random(float timePassed) const;
  //  ANIM_FUNCTION animFunc;
  KeyFrame3D* currentKeyFrame;
  KeyFrame3D* nextKeyFrame;
  tList<KeyFrame3D>* keyFrameList;
  float (Animation3D::*animFunc)(float) const;

  // more class-local description
  PNode* object;
  Vector lastPosition;
  Vector tmpVect;
  float deltaT;
};
